import csv
import os
import sqlite3

from zoran_tools.path_tools import create_father_dir


def readcsvgenerator(filename, encoding):
    """
    读取CSV文件为生成器
    """
    with open(filename, mode='r', encoding=encoding) as f:
        f_csv = csv.reader(f)
        for row in f_csv:
            yield row


def readcsvlist(filename, encoding):
    """
    读取CSV文件为列表
    """
    with open(filename, mode='r', encoding=encoding) as f:
        f_csv = csv.reader(f)
        content = [row for row in f_csv]
    return content
    

def readcsv(filename, encoding='utf8', li=True):
    '''
    接收一个CSV文件名, 返回其内容, 可能返回<list>也可能返回<generator>
    :param filename: <str> 要读取的CSV路径
    :param encoding:
    :param li: True or False, 指定返回的数据格式, 为True时返回列表, 为False时返回生成器
    :return:
    '''
    if li:
        return readcsvlist(filename, encoding)
    else:
        return readcsvgenerator(filename, encoding)


def write_csv(filename, li, quoting=csv.QUOTE_ALL):
    '''
    接收一个文件名(路径)和一个列表, 将列表写入该文件
    :param filename: <str> 要写入的文件名
    :param li: <list> 要写入的列表
    :param quoting: <csv.QUOTE_MINIMAL, csv.QUOTE_ALL, csv.QUOTE_NONNUMERIC, csv.QUOTE_NONE>
        csv.QUOTE_MINIMAL 仅当字段中包含分隔符时才用引号括起,
        csv.QUOTE_ALL 任何情况下都将字段用引号括起,
        csv.QUOTE_NONNUMERIC 括起非数字字段, 数字不括起,
        csv.QUOTE_NONE 不括起任何字段
    '''
    create_father_dir(filename)
    with open(file=filename, mode='a+', encoding='utf8', newline='') as f:
        sp = csv.writer(f, delimiter=',', quotechar='"', quoting=quoting)
        sp.writerow(li)


write_csv_row = write_csv


def write_csv_rows(filename, rows, quoting=True):
    """
    :param filename: <str> 要写入的文件名
    :param rows: <list> 要写入的所有行
    :param quoting: <csv.QUOTE_MINIMAL, csv.QUOTE_ALL, csv.QUOTE_NONNUMERIC, csv.QUOTE_NONE>
        csv.QUOTE_MINIMAL 仅当字段中包含分隔符时才用引号括起,
        csv.QUOTE_ALL 任何情况下都将字段用引号括起,
        csv.QUOTE_NONNUMERIC 括起非数字字段, 数字不括起,
        csv.QUOTE_NONE 不括起任何字段
    """
    create_father_dir(filename)
    if quoting is None:
        quoting = csv.QUOTE_NONE
    elif quoting == 'minimal':
        quoting = csv.QUOTE_MINIMAL
    elif quoting == 'nonnumeric':
        quoting = csv.QUOTE_NONNUMERIC
    else:
        quoting = csv.QUOTE_ALL

    with open(file=filename, mode='a+', encoding='utf8', newline='') as f:
        sp = csv.writer(f, delimiter=',', quotechar='"', quoting=quoting)
        sp.writerows(rows)





__auth__ = 'ChenZhongrun'
__mail__ = 'chenzhongrun@bonc.com.cn'
__cop__ = 'BONC'


from .csv_tools import readcsv, write_csv_row, write_csv_rows
from .path_tools import list_files as listdir, create_father_dir, create_dir_same_as_filename, ask_file, ask_files, ask_dir, chdir
from .path_tools import ZPath, plot_tree
from .csvdb import CsvDB
from .json_tools import jsonp2json
from .zoran_tools import transcode, split_list, split_list_by_len



import csv
import os
import sqlite3

from zoran_tools.path_tools import create_father_dir
from zoran_tools.csv_tools import readcsv


class CsvDB(object):
    def __init__(self, name=None, memory=':memory:'):
        if name:
            self.name = name
        self.db = sqlite3.connect(memory)
        self.table_dict = dict()

    def create_table(self, name, columns):
        """
        根据表名和字段在数据库中创建一个表
        :param name:
        :param columns:
        :return:
        """
        if name not in self.tables:
            sql = 'create table {}({});'.format(name, columns)
            self.db.execute(sql)
            return True
        else:
            return False

    def get_table(self, name=None, filename=None, content=None, fields=None, encoding='utf8'):
        """
        从文件或列表中创建一个Table对象，并在数据库中创建一个表
        """
        table = _Table(db=self, name=name, filename=filename, content=content, fields=fields, encoding=encoding)
        table.create()
        self.table_dict[table.name] = table
        return self

    def table(self, name):
        return self.table_dict[name]

    @property
    def tables(self, printit=False):
        """
        查询数据库所有的表
        """
        sql = 'SELECT name FROM sqlite_master WHERE type="table" ORDER BY name;'
        tables = self.runsql(sql).fetchall()
        if printit:
            print(tables)
        return tables


    def runsql(self, sql):
        """
        运行SQL语句
        :param sql:
        :return:
        """
        return self.db.execute(sql)

    def sql(self, sql):
        """
        运行SQL语句，与self.runsql()一致，另建一个功能重复的函数是为了与Spark中接口命名保持统一
        """
        return self.runsql(sql)

    def run_insert(self, name, content):
        """
        进行表插入
        :param name: 表名
        :param content: 要插入的列表，列表的每一个元素是一个要插入的行
        """
        self.db.executemany(
            'INSERT INTO {} VALUES ({})'.format(name, ','.join(['?']*len(content[0]))),
            [tuple(r) for r in content]
        )
        return len(content)


class _Table(object):
    def __init__(self, db, name=None, filename=None, content=None, fields=None, encoding='utf8'):
        """
        表对象
        :param db: 表所属的数据库
        :param name: 表名
        :param filename: 表内容来自的CSV文件
        :param content: 表内容来自的列表
        :param field: 表字段
        :param encoding: 
        """
        if (not name and not filename) or (not filename and not content):
            raise ValueError
        self.db = db
        self._name = name
        self._filename = filename
        self._content = content
        self._fields = fields
        self.encoding = encoding

    @property
    def name(self):
        """
        表名
        """
        if self._name:
            return self._name
        else:
            return os.path.basename(os.path.splitext(self._filename)[0])


    @property
    def fields(self):
        """
        字段
        """
        if self._fields:
            return self._fields
        else:
            pat = '_{} varchar(600)'  # 与Spark生成的字段名保持一致
            row_length = len(self.content[0])
            if row_length == 0:
                raise ValueError
            indexes = list(range(1, row_length + 1))
            fields = ','.join([pat.format(i) for i in indexes])
            return fields

    @property
    def content(self):
        if self._content:
            return self._content
        else:
            return readcsv(filename=self._filename, encoding=self.encoding)

    def collect(self):
        return self.content  # 重复功能是为了与Spark函数保持一致

    def _create(self):
        self.db.create_table(name=self.name, columns=self.fields)

    def _insert(self):
        self.db.run_insert(name=self.name, content=self.content)

    def create(self):
        self._create()
        self._insert()

    def select_all(self):
        sql = 'select * from {};'.format(self.name)
        return self.db.runsql(sql).fetchall()




import re
import json

def jsonp2json(jsonp_text, format='str'):
    """
    将Jsonp格式以Json格式解析出来
    :param jsonp_text: 要解析的Jsonp文本
    :param format: 要解析为的格式, 可以是str或dict
    :return:
    """
    text_wihtout_new_line = re.sub('\s', '', jsonp_text)
    jsn = re.findall('.*\(({.*})\)[;]{0,1}', text_wihtout_new_line)[0]
    jsn_dict = json.loads(jsn)
    if format == 'dict':
        return jsn_dict
    elif format == 'str':
        return json.dumps(jsn_dict, indent=2, ensure_ascii=False)
    else:
        raise ValueError




import math
import codecs


def readfile(file, encoding='utf8'):
    """
    读取文本文件, 返回文本文件内容
    :param file: 要读取的文件名
    :param encoding: 文件编码, 默认为utf8
    :return: 返回文件内容
    """
    with codecs.open(file, mode='r', encoding=encoding) as f:
        return f.read()


def writefile(file, content, mode='w', encoding='utf8'):
    """
    将文本内容写入文件
    :param file: 要写入的文件名
    :param content: 要写入的内容
    :param mode: 写入模式, 默认为w覆盖
    :param encoding: 写入编码, 默认为utf8
    :return:
    """
    with codecs.open(file, mode=mode, encoding=encoding) as f:
        f.write(content)


def transcode(from_file, to_file, from_code='utf8', to_code='GBK'):
    """
    转换文本文件格式
    :param from_file: 待转换的文件名
    :param to_file: 转换后的文件名
    :param from_code: 转换前的文件编码
    :param to_code: 转换后的文件编码
    :return:
    """
    content = readfile(from_file, encoding=from_code)
    writefile(to_file, content, encoding=to_code)
    return '{} ====> {}'.format(from_file, to_file)


def split_list(li, num=8):
    """
    分割列表
    :param li: 要分割的列表
    :param num: 要分割的份数
    :return: 返回分割结果
    """
    return split_list_by_len(li, math.ceil(len(li) / num))


def split_list_by_len(li, n):
    return [li[i: i + n] for i in range(0, len(li), n)]



import sqlite3


def read_SQLite_table(db, table, sql=None, field='*'):
    '''
    返回SQLite中表的查询结果
    :param db: <str> 数据库文件名
    :param table: <str> 要查的表名
    :param sql: <str> 要执行的SQL语句, 不指定的话就生成一个SQL语句
    :param field: <field> 要查询的字段, 不指定就查询所有字段;
        可以接收<list, tuple, str>等类型
    :param list_: <True, False>, 为True时返回列表, 为False时以生成器形式返回
    '''
    if not sql:
        if isinstance(field, (list, tuple)):
            field = ','.join(field)
        sql = 'select {} from {};'.format(field, table)
    with sqlite3.connect(db) as conn:
        cur = conn.cursor()
        cur.execute(sql)
        result = cur.fetchall()
        result = [list(e) for e in result]
        return result


def showtables(db):
    with sqlite3.connect(db) as conn:
        cur = conn.cursor()
        cur.execute('SELECT name FROM sqlite_master WHERE type="table" ORDER BY name;')
        tables = [r for r in cur.fetchall()]
    return tables


def gen_insert_sql(table_name, content, fields=None, filename=None):
    """
    接收一个CSV内容, 生成插入语句
    :param table_name: 要插入的表名
    :param content: list, 要写入的内容
    :param: fields: list, 要写入的字段名, 不输入时默认为所有字段
    :param filename: 要写入的文件名
    :return:
    """
    if fields:
        pat = 'INSERT INTO {}\n      ({})\nVALUES{};\n'
        sql = pat.format(table_name, ','.join(fields),
                         ',\n      '.join(['({})'.format(','.join(['"{}"'.format(f) for f in e])) for e in content]))
    else:
        pat = 'INSERT INTO {}\nVALUES{};\n'
        sql = pat.format(table_name,
                         ',\n      '.join(['({})'.format(','.join(['"{}"'.format(f) for f in e])) for e in content]))
    if filename:
        with open(filename, mode='a+', encoding='utf8') as f:
            f.write(sql)
    return sql
